@@ -207,7 +207,7 @@ RemoveFavoriteIcon.prototype = {
         let app = null;
         if (source instanceof AppDisplay.AppWellIcon) {
             let appSystem = Shell.AppSystem.get_default();
-            app = appSystem.get_app(source.getId());
+            app = appSystem.lookup_app(source.getId());
         } else if (source.metaWindow) {
             let tracker = Shell.WindowTracker.get_default();
             app = tracker.get_window_app(source.metaWindow);
@@ -275,7 +275,7 @@ Dash.prototype = {
 
         this._appSystem.connect('installed-changed', Lang.bind(this, this._queueRedisplay));
         AppFavorites.getAppFavorites().connect('changed', Lang.bind(this, this._queueRedisplay));
-        this._tracker.connect('app-state-changed', Lang.bind(this, this._queueRedisplay));
+        this._appSystem.connect('app-state-changed', Lang.bind(this, this._queueRedisplay));
 
         Main.overview.connect('item-drag-begin',
                               Lang.bind(this, this._onDragBegin));
@@ -330,7 +330,7 @@ Dash.prototype = {
     _onDragMotion: function(dragEvent) {
         let app = null;
         if (dragEvent.source instanceof AppDisplay.AppWellIcon)
-            app = this._appSystem.get_app(dragEvent.source.getId());
+            app = this._appSystem.lookup_app(dragEvent.source.getId());
         else if (dragEvent.source.metaWindow)
             app = this._tracker.get_window_app(dragEvent.source.metaWindow);
         else
@@ -437,6 +437,7 @@ Dash.prototype = {
 
         let oldIconSize = this.iconSize;
         this.iconSize = newIconSize;
+        this.emit('icon-size-changed');
 
         let scale = oldIconSize / newIconSize;
         for (let i = 0; i < iconChildren.length; i++) {
@@ -470,10 +471,7 @@ Dash.prototype = {
     _redisplay: function () {
         let favorites = AppFavorites.getAppFavorites().getFavoriteMap();
 
-        /* hardcode here pending some design about how exactly desktop contexts behave */
-        let contextId = '';
-
-        let running = this._tracker.get_running_apps(contextId);
+        let running = this._appSystem.get_running();
 
         let children = this._box.get_children().filter(function(actor) {
                 return actor._delegate.child &&
@@ -619,12 +617,12 @@ Dash.prototype = {
     handleDragOver : function(source, actor, x, y, time) {
         let app = null;
         if (source instanceof AppDisplay.AppWellIcon)
-            app = this._appSystem.get_app(source.getId());
+            app = this._appSystem.lookup_app(source.getId());
         else if (source.metaWindow)
             app = this._tracker.get_window_app(source.metaWindow);
 
         // Don't allow favoriting of transient apps
-        if (app == null || app.is_transient())
+        if (app == null || app.is_window_backed())
             return DND.DragMotionResult.NO_DROP;
 
         let favorites = AppFavorites.getAppFavorites().getFavorites();
@@ -704,13 +702,13 @@ Dash.prototype = {
     acceptDrop : function(source, actor, x, y, time) {
         let app = null;
         if (source instanceof AppDisplay.AppWellIcon) {
-            app = this._appSystem.get_app(source.getId());
+            app = this._appSystem.lookup_app(source.getId());
         } else if (source.metaWindow) {
             app = this._tracker.get_window_app(source.metaWindow);
         }
 
         // Don't allow favoriting of transient apps
-        if (app == null || app.is_transient()) {
+        if (app == null || app.is_window_backed()) {
             return false;
         }
 
