@@ -24,6 +24,7 @@
 
 #include "shell-a11y.h"
 #include "shell-global.h"
+#include "shell-global-private.h"
 #include "shell-perf-log.h"
 #include "st.h"
 
@@ -32,6 +33,8 @@ extern GType gnome_shell_plugin_get_type (void);
 #define SHELL_DBUS_SERVICE "org.gnome.Shell"
 #define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
 
+static gboolean is_gdm_mode = FALSE;
+
 static void
 shell_dbus_init (gboolean replace)
 {
@@ -114,6 +117,17 @@ shell_dbus_init (gboolean replace)
       g_print ("failed to acquire org.freedesktop.Notifications: %s\n", error->message);
     }
 
+  /* ...and the on-screen keyboard service */
+  if (!dbus_g_proxy_call (bus, "RequestName", &error,
+                          G_TYPE_STRING, "org.gnome.Caribou.Keyboard",
+                          G_TYPE_UINT, DBUS_NAME_FLAG_REPLACE_EXISTING | request_name_flags,
+                          G_TYPE_INVALID,
+                          G_TYPE_UINT, &request_name_result,
+                          G_TYPE_INVALID))
+    {
+      g_print ("failed to acquire org.gnome.Caribou.Keyboard: %s\n", error->message);
+    }
+
   g_object_unref (bus);
 }
 
@@ -466,6 +480,12 @@ GOptionEntry gnome_shell_options[] = {
     N_("Print version"),
     NULL
   },
+  {
+    "gdm-mode", 0, 0, G_OPTION_ARG_NONE,
+    &is_gdm_mode,
+    N_("Mode used by GDM for login screen"),
+    NULL
+  },
   { NULL }
 };
 
@@ -474,6 +494,7 @@ main (int argc, char **argv)
 {
   GOptionContext *ctx;
   GError *error = NULL;
+  ShellSessionType session_type;
   int ecode;
   TpDebugSender *sender;
 
@@ -524,7 +545,7 @@ main (int argc, char **argv)
   /* Disable debug spew from various libraries */
   g_log_set_handler ("Gvc", G_LOG_LEVEL_DEBUG,
                      muted_log_handler, NULL);
-  g_log_set_handler ("GdmUser", G_LOG_LEVEL_DEBUG,
+  g_log_set_handler ("AccountsService", G_LOG_LEVEL_DEBUG,
                      muted_log_handler, NULL);
   g_log_set_handler ("Bluetooth", G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE,
                      muted_log_handler, NULL);
@@ -540,7 +561,12 @@ main (int argc, char **argv)
   g_log_set_default_handler (default_log_handler, sender);
 
   /* Initialize the global object */
-  shell_global_get ();
+  if (is_gdm_mode)
+      session_type = SHELL_SESSION_GDM;
+  else
+      session_type = SHELL_SESSION_USER;
+
+  _shell_global_init ("session-type", session_type, NULL);
 
   ecode = meta_run ();
 
