@@ -19,6 +19,9 @@ GType shell_stage_input_mode_get_type (void) G_GNUC_CONST;
 GType shell_cursor_get_type (void) G_GNUC_CONST;
 #define SHELL_TYPE_CURSOR (shell_cursor_get_type())
 
+GType shell_session_type_get_type (void) G_GNUC_CONST;
+#define SHELL_TYPE_SESSION_TYPE (shell_session_type_get_type())
+
 /* enumerations from "./shell-mobile-providers.h" */
 GType shell_mobile_access_method_type_get_type (void) G_GNUC_CONST;
 #define SHELL_TYPE_MOBILE_ACCESS_METHOD_TYPE (shell_mobile_access_method_type_get_type())
