@@ -57,6 +57,22 @@ extern void _shell_marshal_VOID__INT_INT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
+/* VOID:STRING,STRING,STRING,STRING,BOXED (./shell-marshal.list:8) */
+extern void _shell_marshal_VOID__STRING_STRING_STRING_STRING_BOXED (GClosure     *closure,
+                                                                    GValue       *return_value,
+                                                                    guint         n_param_values,
+                                                                    const GValue *param_values,
+                                                                    gpointer      invocation_hint,
+                                                                    gpointer      marshal_data);
+
+/* VOID:STRING,OBJECT,STRING,BOXED (./shell-marshal.list:9) */
+extern void _shell_marshal_VOID__STRING_OBJECT_STRING_BOXED (GClosure     *closure,
+                                                             GValue       *return_value,
+                                                             guint         n_param_values,
+                                                             const GValue *param_values,
+                                                             gpointer      invocation_hint,
+                                                             gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* ___shell_marshal_MARSHAL_H__ */
