@@ -2,8 +2,8 @@
 /* Generated data (by glib-mkenums) */
 
 #include "st-enum-types.h"
-/* enumerations from "st/st-button.h" */
-#include "st/st-button.h"
+/* enumerations from "./st/st-button.h" */
+#include "./st/st-button.h"
 GType
 st_button_mask_get_type(void) {
   static GType enum_type_id = 0;
@@ -19,8 +19,8 @@ st_button_mask_get_type(void) {
     }
   return enum_type_id;
 }
-/* enumerations from "st/st-table.h" */
-#include "st/st-table.h"
+/* enumerations from "./st/st-table.h" */
+#include "./st/st-table.h"
 GType
 st_table_child_options_get_type(void) {
   static GType enum_type_id = 0;
@@ -38,8 +38,8 @@ st_table_child_options_get_type(void) {
     }
   return enum_type_id;
 }
-/* enumerations from "st/st-texture-cache.h" */
-#include "st/st-texture-cache.h"
+/* enumerations from "./st/st-texture-cache.h" */
+#include "./st/st-texture-cache.h"
 GType
 st_texture_cache_policy_get_type(void) {
   static GType enum_type_id = 0;
@@ -54,8 +54,8 @@ st_texture_cache_policy_get_type(void) {
     }
   return enum_type_id;
 }
-/* enumerations from "st/st-theme-node.h" */
-#include "st/st-theme-node.h"
+/* enumerations from "./st/st-theme-node.h" */
+#include "./st/st-theme-node.h"
 GType
 st_side_get_type(void) {
   static GType enum_type_id = 0;
@@ -136,8 +136,8 @@ st_gradient_type_get_type(void) {
     }
   return enum_type_id;
 }
-/* enumerations from "st/st-types.h" */
-#include "st/st-types.h"
+/* enumerations from "./st/st-types.h" */
+#include "./st/st-types.h"
 GType
 st_align_get_type(void) {
   static GType enum_type_id = 0;
@@ -169,8 +169,8 @@ st_icon_type_get_type(void) {
     }
   return enum_type_id;
 }
-/* enumerations from "st/st-widget.h" */
-#include "st/st-widget.h"
+/* enumerations from "./st/st-widget.h" */
+#include "./st/st-widget.h"
 GType
 st_text_direction_get_type(void) {
   static GType enum_type_id = 0;
